@@ -1,6 +1,6 @@
 /*
  *  GTetrinet
- *  Copyright (C) 1999, 2000  Ka-shu Wong (kswong@zip.com.au)
+ *  Copyright (C) 1999, 2000, 2001, 2002, 2003  Ka-shu Wong (kswong@zip.com.au)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,9 +16,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+
+#ifdef HAVE_CONFIG_H
+# include "../config.h"
+#endif
+
 #include <gtk/gtk.h>
 #include <gnome.h>
-#include <gdk_imlib.h>
 #include <stdio.h>
 
 #include "config.h"
@@ -27,6 +31,8 @@
 #include "tetris.h"
 #include "fields.h"
 #include "misc.h"
+#include "gtetrinet.h"
+#include "string.h"
 
 #define BLOCKSIZE bsize
 #define SMALLBLOCKSIZE (BLOCKSIZE/2)
@@ -38,30 +44,43 @@ static GtkWidget *fieldwidgets[6], *nextpiecewidget, *fieldlabels[6][6],
 static GtkWidget *fields_page_contents (void);
 
 static gint fields_expose_event (GtkWidget *widget, GdkEventExpose *event, int field);
-static gint fields_nextpiece_expose (GtkWidget *widget, GdkEventExpose *event);
-static gint fields_specials_expose (GtkWidget *widget, GdkEventExpose *event);
+static gint fields_nextpiece_expose (GtkWidget *widget);
+static gint fields_specials_expose (GtkWidget *widget);
 
 static void fields_refreshfield (int field);
 static void fields_drawblock (int field, int x, int y, char block);
 
-static gint fields_eatkey (GtkWidget *widget, GdkEventKey *key);
+static void gmsginput_activate (void);
 
 static GdkPixmap *blockpix;
 
+static GdkColor black = {0, 0, 0, 0};
+static GdkBitmap *bitmap;
+static GdkCursor *invisible_cursor, *arrow_cursor;
+
 static FIELD displayfields[6]; /* what is actually displayed */
 static TETRISBLOCK displayblock;
 
 void fields_init (void)
 {
-    GdkBitmap *bmap;
     GtkWidget *mb;
-    if (gdk_imlib_load_file_to_pixmap (blocksfile, &blockpix, &bmap) == 0) {
-        mb = gnome_message_box_new (_("Error loading theme: cannot load graphics file\n"
-                                      "Falling back to default"),
-                                    GNOME_MESSAGE_BOX_ERROR, GNOME_STOCK_BUTTON_OK, NULL);
-        gnome_dialog_run (GNOME_DIALOG(mb));
+    GdkPixbuf *pb = NULL;
+    GError *err = NULL;
+    GdkBitmap *mask = NULL;
+    
+    if (!(pb = gdk_pixbuf_new_from_file(blocksfile, &err))) {
+        mb = gtk_message_dialog_new (NULL,
+                                     GTK_DIALOG_MODAL,
+                                     GTK_MESSAGE_ERROR,
+                                     GTK_BUTTONS_OK,
+                                     _("Error loading theme: cannot load graphics file\n"
+                                       "Falling back to default"));
+        gtk_dialog_run (GTK_DIALOG (mb));
+        gtk_widget_destroy (mb);
+	g_string_assign(currenttheme, DEFAULTTHEME);
         config_loadtheme (DEFAULTTHEME);
-        if (gdk_imlib_load_file_to_pixmap (blocksfile, &blockpix, &bmap) == 0) {
+        err = NULL;
+        if (!(pb = gdk_pixbuf_new_from_file(blocksfile, &err))) {
             /* shouldnt happen */
             fprintf (stderr, _("Error loading default theme: Aborting...\n"
                                "Check for installation errors\n"));
@@ -69,11 +88,13 @@ void fields_init (void)
         }
     }
     /* we dont want the bitmap mask */
+    gdk_pixbuf_render_pixmap_and_mask(pb, &blockpix, &mask, 1);
+    gdk_pixbuf_unref(pb);
 }
 
 void fields_cleanup (void)
 {
-    gdk_imlib_free_pixmap (blockpix);
+  g_object_unref(blockpix);
 }
 
 /* a mess of functions here for creating the fields page */
@@ -83,11 +104,16 @@ GtkWidget *fields_page_new (void)
     pagecontents = fields_page_contents ();
 
     if (fieldspage == NULL) {
-        fieldspage = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-        gtk_container_border_width (GTK_CONTAINER(fieldspage), 2);
+        fieldspage = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
+        gtk_container_set_border_width (GTK_CONTAINER(fieldspage), 2);
     }
     gtk_container_add (GTK_CONTAINER(fieldspage), pagecontents);
 
+    /* create the cursors */
+    bitmap = gdk_bitmap_create_from_data (GTK_WIDGET (fieldspage)->window, "\0", 1, 1);
+    invisible_cursor = gdk_cursor_new_from_pixmap (bitmap, bitmap, &black, &black, 0, 0);
+    arrow_cursor = gdk_cursor_new (GDK_X_CURSOR);
+
     return fieldspage;
 }
 
@@ -101,8 +127,9 @@ void fields_page_destroy_contents (void)
 
 GtkWidget *fields_page_contents (void)
 {
-    GtkWidget *table, *widget, *align, *border, *box, *table2, *hbox;
+    GtkWidget *vbox, *table, *widget, *align, *border, *box, *table2, *hbox, *scroll;
     table = gtk_table_new (4, 5, FALSE);
+    vbox = gtk_vbox_new (FALSE, 0);
 
     gtk_table_set_row_spacings (GTK_TABLE(table), 2);
     gtk_table_set_col_spacings (GTK_TABLE(table), 2);
@@ -142,175 +169,181 @@ GtkWidget *fields_page_contents (void)
             gtk_box_pack_start (GTK_BOX(hbox), fieldlabels[i][3], TRUE, TRUE, 0);
             gtk_box_pack_start (GTK_BOX(hbox), fieldlabels[i][4], FALSE, FALSE, 2);
             gtk_box_pack_start (GTK_BOX(hbox), fieldlabels[i][5], FALSE, FALSE, 0);
-            gtk_widget_show (hbox);
 
             fields_setlabel (i, NULL, NULL, 0);
 
             widget = gtk_event_box_new ();
             gtk_container_add (GTK_CONTAINER(widget), hbox);
-            gtk_widget_set_usize (widget, blocksize * FIELDWIDTH, 0);
-            gtk_widget_show (widget);
-            gtk_box_pack_start (GTK_BOX(box), widget, FALSE, FALSE, 0);
+            gtk_widget_set_size_request (widget, blocksize * FIELDWIDTH, -1);
+            gtk_box_pack_start (GTK_BOX(box), widget, TRUE, TRUE, 0);
             /* the field */
             fieldwidgets[i] = gtk_drawing_area_new ();
+            
             /* attach the signals */
-            gtk_signal_connect (GTK_OBJECT(fieldwidgets[i]), "expose_event",
+            g_signal_connect (G_OBJECT(fieldwidgets[i]), "expose_event",
                                 GTK_SIGNAL_FUNC(fields_expose_event), (gpointer)i);
             gtk_widget_set_events (fieldwidgets[i], GDK_EXPOSURE_MASK);
             /* set the size */
-            gtk_drawing_area_size (GTK_DRAWING_AREA(fieldwidgets[i]),
-                                   blocksize * FIELDWIDTH,
-                                   blocksize * FIELDHEIGHT);
-            gtk_widget_show (fieldwidgets[i]);
-            gtk_box_pack_start (GTK_BOX(box), fieldwidgets[i], FALSE, FALSE, 0);
-            gtk_widget_show (box);
+            gtk_widget_set_size_request (fieldwidgets[i],
+                                         blocksize * FIELDWIDTH,
+                                         blocksize * FIELDHEIGHT);
+            gtk_box_pack_start (GTK_BOX(box), fieldwidgets[i], TRUE, TRUE, 0);
             border = gtk_frame_new (NULL);
             gtk_frame_set_shadow_type (GTK_FRAME(border), GTK_SHADOW_IN);
             gtk_container_add (GTK_CONTAINER(border), box);
-            gtk_widget_show (border);
             /* align it */
             align = gtk_alignment_new (0.5, valign, 0.0, 0.0);
             gtk_container_add (GTK_CONTAINER(align), border);
-            gtk_widget_show (align);
-            gtk_table_attach_defaults (GTK_TABLE(table), align,
-                                       p[i][0], p[i][1], p[i][2], p[i][3]);
+            gtk_table_attach (GTK_TABLE(table), align,
+                              p[i][0], p[i][1], p[i][2], p[i][3],
+                              GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
+                              0, 0);
         }
     }
     /* next block thingy */
     box = gtk_vbox_new (FALSE, 2);
 
     widget = leftlabel_new (_("Next piece:"));
-    gtk_widget_show (widget);
-    gtk_box_pack_start (GTK_BOX(box), widget, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX(box), widget, TRUE, TRUE, 0);
     /* box that displays the next block */
     border = gtk_frame_new (NULL);
     gtk_frame_set_shadow_type (GTK_FRAME(border), GTK_SHADOW_IN);
     nextpiecewidget = gtk_drawing_area_new ();
-    gtk_signal_connect (GTK_OBJECT(nextpiecewidget), "expose_event",
+    g_signal_connect (G_OBJECT(nextpiecewidget), "expose_event",
                         GTK_SIGNAL_FUNC(fields_nextpiece_expose), NULL);
     gtk_widget_set_events (nextpiecewidget, GDK_EXPOSURE_MASK);
-    gtk_drawing_area_size (GTK_DRAWING_AREA(nextpiecewidget), BLOCKSIZE*9/2, BLOCKSIZE*9/2);
-    gtk_widget_show (nextpiecewidget);
+    gtk_widget_set_size_request (nextpiecewidget, BLOCKSIZE*9/2, BLOCKSIZE*9/2);
     gtk_container_add (GTK_CONTAINER(border), nextpiecewidget);
-    gtk_widget_show (border);
     align = gtk_alignment_new (0.5, 0.5, 0, 0);
     gtk_container_add (GTK_CONTAINER(align), border);
-    gtk_widget_show (align);
-    gtk_box_pack_start (GTK_BOX(box), align, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX(box), align, TRUE, TRUE, 0);
     /* lines, levels and stuff */
     table2 = gtk_table_new (4, 2, FALSE);
     gtk_table_set_col_spacings (GTK_TABLE(table2), 5);
     widget = leftlabel_new (_("Lines:"));
-    gtk_widget_show (widget);
     gtk_table_attach_defaults (GTK_TABLE(table2), widget, 0, 1, 0, 1);
     widget = gtk_label_new ("");
-    gtk_widget_show (widget);
     gtk_table_attach_defaults (GTK_TABLE(table2), widget, 0, 1, 1, 2);
     widget = leftlabel_new (_("Level:"));
-    gtk_widget_show (widget);
     gtk_table_attach_defaults (GTK_TABLE(table2), widget, 0, 1, 2, 3);
     activelabel = leftlabel_new (_("Active level:"));
-    gtk_widget_show (activelabel);
     gtk_table_attach_defaults (GTK_TABLE(table2), activelabel, 0, 1, 3, 4);
     lineswidget = leftlabel_new ("");
-    gtk_widget_show (lineswidget);
     gtk_table_attach_defaults (GTK_TABLE(table2), lineswidget, 1, 2, 0, 1);
     widget = gtk_label_new ("");
-    gtk_widget_show (widget);
     gtk_table_attach_defaults (GTK_TABLE(table2), widget, 1, 2, 1, 2);
     levelwidget = leftlabel_new ("");
-    gtk_widget_show (levelwidget);
     gtk_table_attach_defaults (GTK_TABLE(table2), levelwidget, 1, 2, 2, 3);
     activewidget = leftlabel_new ("");
-    gtk_widget_show (activewidget);
     gtk_table_attach_defaults (GTK_TABLE(table2), activewidget, 1, 2, 3, 4);
-    gtk_widget_show (table2);
-    gtk_box_pack_start (GTK_BOX(box), table2, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX(box), table2, TRUE, TRUE, 0);
 
-    gtk_widget_show (box);
     /* align it */
     align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-    gtk_widget_set_usize (align, BLOCKSIZE*6, BLOCKSIZE*11);
+    gtk_widget_set_size_request (align, BLOCKSIZE*6, BLOCKSIZE*11);
     gtk_container_add (GTK_CONTAINER(align), box);
-    gtk_widget_show (align);
-    gtk_table_attach_defaults (GTK_TABLE(table), align, 1, 2, 0, 1);
+    gtk_table_attach (GTK_TABLE(table), align, 1, 2, 0, 1,
+                      GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
+                      0, 0);
 
     /* the specials thingy */
     box = gtk_hbox_new (FALSE, 0);
     speciallabel = gtk_label_new ("");
     gtk_widget_show (speciallabel);
     fields_setspeciallabel (NULL);
-    gtk_box_pack_start (GTK_BOX(box), speciallabel, FALSE, FALSE, 0);
+    align = gtk_alignment_new (1.0, 0.0, 1.0, 1.0);
+    gtk_container_add (GTK_CONTAINER (align), speciallabel);
+    gtk_box_pack_start (GTK_BOX(box), align, TRUE, TRUE, 0);
     border = gtk_frame_new (NULL);
     gtk_frame_set_shadow_type (GTK_FRAME(border), GTK_SHADOW_IN);
     specialwidget = gtk_drawing_area_new ();
-    gtk_signal_connect (GTK_OBJECT(specialwidget), "expose_event",
+    g_signal_connect (G_OBJECT(specialwidget), "expose_event",
                         GTK_SIGNAL_FUNC(fields_specials_expose), NULL);
-    gtk_drawing_area_size (GTK_DRAWING_AREA(specialwidget), BLOCKSIZE*18, BLOCKSIZE);
+    gtk_widget_set_size_request (specialwidget, BLOCKSIZE*18, BLOCKSIZE);
     gtk_widget_show (specialwidget);
     gtk_container_add (GTK_CONTAINER(border), specialwidget);
-    gtk_widget_show (border);
     gtk_box_pack_end (GTK_BOX(box), border, FALSE, FALSE, 0);
-    gtk_widget_show (box);
-    gtk_widget_set_usize (box, BLOCKSIZE*24, 0);
+    gtk_widget_set_size_request (box, BLOCKSIZE*24, -1);
     /* align it */
     align = gtk_alignment_new (0.5, 1.0, 0.7, 0.0);
     gtk_container_add (GTK_CONTAINER(align), box);
-    gtk_widget_show (align);
-    gtk_table_attach_defaults (GTK_TABLE (table), align, 0, 3, 2, 3);
+    gtk_table_attach (GTK_TABLE (table), align, 0, 3, 2, 3,
+                      GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
+                      0, 0);
 
     /* attacks and defenses */
 
     box = gtk_vbox_new (FALSE, 0);
     widget = gtk_label_new (_("Attacks and defenses:"));
-    gtk_widget_show (widget);
-    gtk_box_pack_start (GTK_BOX(box), widget, FALSE, FALSE, 0);
-    attdefwidget = gtk_text_new (NULL, NULL);
-    gtk_widget_set_usize (attdefwidget, BLOCKSIZE*12, BLOCKSIZE*10);
-    gtk_text_set_word_wrap (GTK_TEXT(attdefwidget), TRUE);
+    gtk_box_pack_start (GTK_BOX(box), widget, TRUE, TRUE, 0);
+    attdefwidget = gtk_text_view_new_with_buffer(gtk_text_buffer_new(tag_table));
+    gtk_widget_set_size_request (attdefwidget, MAX(22*12, BLOCKSIZE*12), BLOCKSIZE*10);
+    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(attdefwidget), GTK_WRAP_WORD);
     GTK_WIDGET_UNSET_FLAGS (attdefwidget, GTK_CAN_FOCUS);
-    gtk_widget_show (attdefwidget);
-    gtk_box_pack_start (GTK_BOX(box), attdefwidget, FALSE, FALSE, 0);
-    gtk_widget_show (box);
+    scroll = gtk_scrolled_window_new (NULL, NULL);
+    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scroll),
+                                    GTK_POLICY_AUTOMATIC,
+                                    GTK_POLICY_AUTOMATIC);
+    gtk_container_add (GTK_CONTAINER(scroll), attdefwidget);
+    gtk_box_pack_start (GTK_BOX(box), scroll, TRUE, TRUE, 0);
     align = gtk_alignment_new (0.5, 0.5, 0.5, 0.0);
     gtk_container_add (GTK_CONTAINER(align), box);
-    gtk_widget_show (align);
-    gtk_table_attach_defaults (GTK_TABLE(table), align, 1, 3, 1, 2);
+    gtk_table_attach (GTK_TABLE(table), align, 1, 3, 1, 2,
+                      GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
+                      0, 0);
 
     /* game messages */
     table2 = gtk_table_new (1, 2, FALSE);
-    gmsgtext = gtk_text_new (NULL, NULL);
-    gtk_widget_set_usize (gmsgtext, 0, 46);
-    gtk_widget_show (gmsgtext);
+    gmsgtext = gtk_text_view_new_with_buffer(gtk_text_buffer_new(tag_table));
+    gtk_widget_set_size_request (gmsgtext, -1, 48);
     GTK_WIDGET_UNSET_FLAGS (gmsgtext, GTK_CAN_FOCUS);
-    gtk_table_attach (GTK_TABLE(table2), gmsgtext, 0, 1, 0, 1,
-                      GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_SHRINK,
+    scroll = gtk_scrolled_window_new (NULL, NULL);
+    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scroll),
+                                    GTK_POLICY_AUTOMATIC,
+                                    GTK_POLICY_AUTOMATIC);
+    gtk_container_add (GTK_CONTAINER(scroll), gmsgtext);
+    gtk_table_attach (GTK_TABLE(table2), scroll, 0, 1, 0, 1,
+                      GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
                       0, 0);
-    gmsginput = gtk_entry_new_with_max_length (128);
-    gtk_widget_show (gmsginput);
+    gmsginput = gtk_entry_new ();
+    gtk_entry_set_max_length (GTK_ENTRY (gmsginput), 128);
     /* eat up key messages */
-    gtk_signal_connect (GTK_OBJECT(gmsginput), "key_press_event",
-                        GTK_SIGNAL_FUNC(fields_eatkey), NULL);
+    g_signal_connect (G_OBJECT(gmsginput), "activate",
+                        GTK_SIGNAL_FUNC(gmsginput_activate), NULL);
     gtk_table_attach (GTK_TABLE(table2), gmsginput, 0, 1, 1, 2,
                       GTK_FILL | GTK_EXPAND, 0, 0, 0);
-    gtk_widget_show (table2);
-    gtk_widget_set_usize (table2, 0, 48);
-    gtk_table_attach_defaults (GTK_TABLE(table), table2, 0, 5, 3, 4);
+    gtk_widget_set_size_request (table2, -1, 70);
+    
+    align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+    gtk_container_add (GTK_CONTAINER (align), table);
+    gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 0);
+    align = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
+    gtk_container_add (GTK_CONTAINER (align), table2);
+    gtk_box_pack_start (GTK_BOX (vbox), align, TRUE, TRUE, 0);
+/*    gtk_table_attach (GTK_TABLE(table), table2, 0, 5, 3, 4,
+                      GTK_FILL | GTK_EXPAND, 0, 0, 0);*/
 
-    gtk_widget_show (table);
+    gtk_widget_show_all (vbox);
 
     fields_setlines (-1);
     fields_setlevel (-1);
     fields_setactivelevel (-1);
     fields_gmsginput (FALSE);
 
-    return table;
+    return vbox;
 }
 
 
 gint fields_expose_event (GtkWidget *widget, GdkEventExpose *event, int field)
 {
+    widget = widget;
+    event = event;
     fields_refreshfield (field);
+    /* hide the cursor */
+    if (ingame)
+      gdk_window_set_cursor (widget->window, invisible_cursor);
+    else
+      gdk_window_set_cursor (widget->window, arrow_cursor);
 
     return FALSE;
 }
@@ -363,17 +396,18 @@ void fields_drawblock (int field, int x, int y, char block)
     destx = blocksize * x;
     desty = blocksize * y;
 
-    gdk_draw_pixmap (fieldwidgets[field]->window,
-                     fieldwidgets[field]->style->black_gc,
-                     blockpix, srcx, srcy, destx, desty,
-                     blocksize, blocksize);
+    gdk_draw_drawable (fieldwidgets[field]->window,
+                       fieldwidgets[field]->style->black_gc,
+                       blockpix, srcx, srcy, destx, desty,
+                       blocksize, blocksize);
 }
 
 void fields_setlabel (int field, char *name, char *team, int num)
 {
-    char buf[10];
+    char buf[11];
+    gchar *name_utf8, *team_utf8;
 
-    sprintf (buf, "%d", num);
+    g_snprintf (buf, sizeof(buf), "%d", num);
     
     if (name == NULL) {
         gtk_widget_hide (fieldlabels[field][0]);
@@ -382,51 +416,63 @@ void fields_setlabel (int field, char *name, char *team, int num)
         gtk_widget_show (fieldlabels[field][3]);
         gtk_widget_hide (fieldlabels[field][4]);
         gtk_widget_hide (fieldlabels[field][5]);
-        gtk_label_set (GTK_LABEL(fieldlabels[field][0]), "");
-        gtk_label_set (GTK_LABEL(fieldlabels[field][2]), "");
-        gtk_label_set (GTK_LABEL(fieldlabels[field][3]), _("Not playing"));
-        gtk_label_set (GTK_LABEL(fieldlabels[field][5]), "");
+        gtk_label_set_text (GTK_LABEL(fieldlabels[field][0]), "");
+        gtk_label_set_text (GTK_LABEL(fieldlabels[field][2]), "");
+        gtk_label_set_text (GTK_LABEL(fieldlabels[field][3]), _("Not playing"));
+        gtk_label_set_text (GTK_LABEL(fieldlabels[field][5]), "");
     }
     else {
+        name_utf8 = g_locale_to_utf8 (nocolor (name), -1, NULL, NULL, NULL);
         gtk_widget_show (fieldlabels[field][0]);
         gtk_widget_show (fieldlabels[field][1]);
         gtk_widget_show (fieldlabels[field][2]);
         gtk_widget_hide (fieldlabels[field][3]);
-        gtk_label_set (GTK_LABEL(fieldlabels[field][0]), buf);
-        gtk_label_set (GTK_LABEL(fieldlabels[field][2]), nocolor(name));
-        gtk_label_set (GTK_LABEL(fieldlabels[field][3]), "");
+        gtk_label_set_text (GTK_LABEL(fieldlabels[field][0]), buf);
+        gtk_label_set_text (GTK_LABEL(fieldlabels[field][2]), name_utf8);
+        gtk_label_set_text (GTK_LABEL(fieldlabels[field][3]), "");
         if (team == NULL || team[0] == 0) {
             gtk_widget_hide (fieldlabels[field][4]);
             gtk_widget_hide (fieldlabels[field][5]);
-            gtk_label_set (GTK_LABEL(fieldlabels[field][5]), "");
+            gtk_label_set_text (GTK_LABEL(fieldlabels[field][5]), "");
         }
         else {
+            team_utf8 = g_locale_to_utf8 (nocolor (team), -1, NULL, NULL, NULL);
             gtk_widget_show (fieldlabels[field][4]);
             gtk_widget_show (fieldlabels[field][5]);
-            gtk_label_set (GTK_LABEL(fieldlabels[field][5]), nocolor(team));
+            gtk_label_set_text (GTK_LABEL(fieldlabels[field][5]), team_utf8);
+            g_free (team_utf8);
         }
+        g_free (name_utf8);
     }
 }
 
 void fields_setspeciallabel (char *label)
 {
     if (label == NULL) {
-        gtk_label_set (GTK_LABEL(speciallabel), _("Specials:"));
+        gtk_label_set_text (GTK_LABEL(speciallabel), _("Specials:"));
     }
     else {
-        gtk_label_set (GTK_LABEL(speciallabel), label);
+        gtk_label_set_text (GTK_LABEL(speciallabel), label);
     }
 }
 
-gint fields_nextpiece_expose (GtkWidget *widget, GdkEventExpose *event)
+gint fields_nextpiece_expose (GtkWidget *widget)
 {
     fields_drawnextblock (NULL);
+    if (ingame)
+      gdk_window_set_cursor (widget->window, invisible_cursor);
+    else
+      gdk_window_set_cursor (widget->window, arrow_cursor);
     return FALSE;
 }
 
-gint fields_specials_expose (GtkWidget *widget, GdkEventExpose *event)
+gint fields_specials_expose (GtkWidget *widget)
 {
     fields_drawspecials ();
+    if (ingame)
+      gdk_window_set_cursor (widget->window, invisible_cursor);
+    else
+      gdk_window_set_cursor (widget->window, arrow_cursor);
     return FALSE;
 }
 
@@ -435,10 +481,10 @@ void fields_drawspecials (void)
     int i;
     for (i = 0; i < 18; i ++) {
         if (i < specialblocknum) {
-            gdk_draw_pixmap (specialwidget->window,
-                             specialwidget->style->black_gc,
-                             blockpix, (specialblocks[i]-1)*BLOCKSIZE,
-                             0, BLOCKSIZE*i, 0, BLOCKSIZE, BLOCKSIZE);
+            gdk_draw_drawable (specialwidget->window,
+                               specialwidget->style->black_gc,
+                               blockpix, (specialblocks[i]-1)*BLOCKSIZE,
+                               0, BLOCKSIZE*i, 0, BLOCKSIZE, BLOCKSIZE);
         }
         else {
             gdk_draw_rectangle (specialwidget->window, specialwidget->style->black_gc,
@@ -463,12 +509,12 @@ void fields_drawnextblock (TETRISBLOCK block)
     for (y = ystart; y < 4; y ++)
         for (x = xstart; x < 4; x ++) {
             if (block[y][x]) {
-                gdk_draw_pixmap (nextpiecewidget->window,
-                                 nextpiecewidget->style->black_gc,
-                                 blockpix, (block[y][x]-1)*BLOCKSIZE, 0,
-                                 BLOCKSIZE*(x-xstart)+BLOCKSIZE/4,
-                                 BLOCKSIZE*(y-ystart)+BLOCKSIZE/4,
-                                 BLOCKSIZE, BLOCKSIZE);
+                gdk_draw_drawable (nextpiecewidget->window,
+                                   nextpiecewidget->style->black_gc,
+                                   blockpix, (block[y][x]-1)*BLOCKSIZE, 0,
+                                   BLOCKSIZE*(x-xstart)+BLOCKSIZE/4,
+                                   BLOCKSIZE*(y-ystart)+BLOCKSIZE/4,
+                                   BLOCKSIZE, BLOCKSIZE);
             }
         }
     memcpy (displayblock, block, 16);
@@ -476,23 +522,32 @@ void fields_drawnextblock (TETRISBLOCK block)
 
 void fields_attdefmsg (char *text)
 {
-    textbox_addtext (GTK_TEXT(attdefwidget), text);
-    adjust_bottom (GTK_TEXT(attdefwidget)->vadj);
+    textbox_addtext (GTK_TEXT_VIEW(attdefwidget), text);
+    adjust_bottom_text_view (GTK_TEXT_VIEW(attdefwidget));
+}
+
+void fields_attdeffmt (const char *fmt, ...)
+{
+    va_list ap;
+    char *text = NULL;
+
+    va_start(ap, fmt);
+    text = g_strdup_vprintf(fmt,ap);
+    va_end(ap);
+
+    fields_attdefmsg (text); g_free(text);
 }
 
 void fields_attdefclear (void)
 {
-    gtk_text_freeze (GTK_TEXT(attdefwidget));
-    gtk_text_backward_delete (GTK_TEXT(attdefwidget),
-                              gtk_text_get_point (GTK_TEXT(attdefwidget)));
-    gtk_text_thaw (GTK_TEXT(attdefwidget));
+  gtk_text_buffer_set_text(GTK_TEXT_VIEW(attdefwidget)->buffer, "", 0);
 }
 
 void fields_setlines (int l)
 {
     char buf[16] = "";
     if (l >= 0)
-        sprintf (buf, "%d", l);
+        g_snprintf (buf, sizeof(buf), "%d", l);
     leftlabel_set (lineswidget, buf);
 }
 
@@ -500,7 +555,7 @@ void fields_setlevel (int l)
 {
     char buf[16] = "";
     if (l > 0)
-        sprintf (buf, "%d", l);
+        g_snprintf (buf, sizeof(buf), "%d", l);
     leftlabel_set (levelwidget, buf);
 }
 
@@ -512,28 +567,25 @@ void fields_setactivelevel (int l)
         gtk_widget_hide (activewidget);
     }
     else {
-        sprintf (buf, "%d", l);
+        g_snprintf (buf, sizeof(buf), "%d", l);
         leftlabel_set (activewidget, buf);
         gtk_widget_show (activelabel);
         gtk_widget_show (activewidget);
     }
 }
 
-void fields_gmsgadd (char *str)
+void fields_gmsgadd (const char *str)
 {
-    textbox_addtext (GTK_TEXT(gmsgtext), str);
-    adjust_bottom (GTK_TEXT(gmsgtext)->vadj);
+    textbox_addtext (GTK_TEXT_VIEW(gmsgtext), str);
+    adjust_bottom_text_view (GTK_TEXT_VIEW(gmsgtext));
 }
 
 void fields_gmsgclear (void)
 {
-    gtk_text_freeze (GTK_TEXT(gmsgtext));
-    gtk_text_backward_delete (GTK_TEXT(gmsgtext),
-                              gtk_text_get_point (GTK_TEXT(gmsgtext)));
-    gtk_text_thaw (GTK_TEXT(gmsgtext));
+  gtk_text_buffer_set_text(GTK_TEXT_VIEW(gmsgtext)->buffer, "", 0);
 }
 
-void fields_gmsginput (int i)
+void fields_gmsginput (gboolean i)
 {
     if (i) {
         gtk_widget_show (gmsginput);
@@ -544,41 +596,59 @@ void fields_gmsginput (int i)
 
 void fields_gmsginputclear (void)
 {
-    gtk_entry_set_text (GTK_ENTRY(gmsginput), "");
-    gtk_entry_set_position (GTK_ENTRY(gmsginput), 0);
+    gtk_entry_set_text (GTK_ENTRY (gmsginput), "");
+    gtk_editable_set_position (GTK_EDITABLE (gmsginput), 0);
 }
 
 void fields_gmsginputactivate (int t)
 {
     if (t)
+    {
+        fields_gmsginputclear ();
         gtk_widget_grab_focus (gmsginput);
+    }
     else
-        /* do nothing */;
+        { /* do nothing */; }
 }
 
-void fields_gmsginputadd (char *c)
+void gmsginput_activate (void)
 {
-    gtk_entry_append_text (GTK_ENTRY(gmsginput), c);
-    gtk_entry_set_position (GTK_ENTRY(gmsginput),
-                            strlen(gtk_entry_get_text(GTK_ENTRY(gmsginput))));
+    gchar *locale_s, buf[256];
+    const gchar *s;
+  
+    if (gmsgstate == 0)
+    {
+        fields_gmsginputclear ();
+        return;
+    }
+    s = fields_gmsginputtext ();
+    if (strlen(s) > 0) {
+        if (strncmp("/me ", s, 4) == 0) {
+            /* post /me thingy */
+            g_snprintf (buf, sizeof(buf), "* %s %s", nick, s+4);
+            locale_s = g_locale_from_utf8 (buf, -1, NULL, NULL, NULL);
+			/* FIXME : if there is an error while converting from UTF8 to current locale, we ignore the message */
+			if (locale_s == NULL) return;
+            client_outmessage (OUT_GMSG, locale_s);
+            g_free (locale_s);
+        }
+        else {
+            /* post message */
+            g_snprintf (buf, sizeof(buf), "<%s> %s", nick, s);
+            locale_s = g_locale_from_utf8 (buf, -1, NULL, NULL, NULL);
+			/* FIXME : if there is an error while converting from UTF8 to current locale, we ignore the message */
+			if (locale_s == NULL) return;
+            client_outmessage (OUT_GMSG, locale_s);
+            g_free (locale_s);
+        }
+    }
+    fields_gmsginputclear ();
+    fields_gmsginput (FALSE);
+    unblock_keyboard_signal ();
+    gmsgstate = 0;
 }
 
-void fields_gmsginputback (void)
-{
-    char buf[256];
-    strcpy (buf, gtk_entry_get_text(GTK_ENTRY(gmsginput)));
-    buf[strlen(buf)-1] = 0;
-    gtk_entry_set_text (GTK_ENTRY(gmsginput), buf);
-    gtk_entry_set_position (GTK_ENTRY(gmsginput), strlen(buf));
-}
-
-char *fields_gmsginputtext (void)
+const char *fields_gmsginputtext (void)
 {
     return gtk_entry_get_text (GTK_ENTRY(gmsginput));
 }
-
-gint fields_eatkey (GtkWidget *widget, GdkEventKey *key)
-{
-    gtk_signal_emit_stop_by_name (GTK_OBJECT(widget), "key_press_event");
-    return TRUE;
-}
