@@ -185,6 +185,9 @@ void tetrinet_inmessage (enum inmsg_type msgtype, char *data)
     case IN_CONNECT:
         list_issued = 0;
         up_chan_list_source = g_timeout_add (30000, (GSourceFunc) partyline_update_channel_list, NULL);
+        partyline_joining_channel ("");
+        show_start_button ();
+        show_disconnect_button ();
         break;
     case IN_DISCONNECT:
         if (!connected) {
@@ -212,6 +215,7 @@ void tetrinet_inmessage (enum inmsg_type msgtype, char *data)
                        TETRI_TB_C_DARK_GREEN, TETRI_TB_BOLD);
         partyline_clear_list_channel ();
         partyline_joining_channel (NULL);
+        show_connect_button ();
         break;
     case IN_CONNECTERROR:
     connecterror:
@@ -229,6 +233,7 @@ void tetrinet_inmessage (enum inmsg_type msgtype, char *data)
             gtk_dialog_run (GTK_DIALOG(dialog));
             gtk_widget_destroy (dialog);
             g_free (data_utf8);
+            show_connect_button ();
         }
         break;
     case IN_PLAYERNUM:
@@ -280,6 +285,7 @@ void tetrinet_inmessage (enum inmsg_type msgtype, char *data)
             }
         }
         /* show partyline on successful connect */
+        partyline_update_channel_list ();
         show_partyline_page ();
         break;
     case IN_PLAYERJOIN:
@@ -399,62 +405,83 @@ void tetrinet_inmessage (enum inmsg_type msgtype, char *data)
               break;
             token = strtok (NULL, "");
             if (token == NULL) token = "";
+            //printf ("* %s\n", token);
             if (pnum == 0) {
                 if (strncmp (token, "\04\04\04\04\04\04\04\04", 8) == 0) {
                     /* tetrix identification string */
                     tetrix = TRUE;
                 }
-                else if (tetrix) {
-                    gchar *line = nocolor (token);
+                else
+                {
+                  gchar *line = nocolor (token);
+                  gchar *aux;
                       
-                    if (list_issued > 0)
+                  if (list_enabled && (list_issued > 0))
+                  {
+                    if (*line == '(')
                     {
-                      if (*line == '(')
-                      {
-                        partyline_add_channel (line);
-                        break;
-                      }
-                      
-                      if (!strncmp ("List", line, 4))
-                      {
-                        partyline_more_channel_lines ();
-                        break;
-                      }
-                      
-                      if (!strncmp ("TetriNET", line, 8))
-                        break;
-                      
-                      if (!strncmp ("You", line, 3))
-                      {
-                        /* we will use the error message as list stopper */
-                        list_issued--;
-                        if (list_issued <= 0)
-                          stop_list();
-                        break;
-                      }
-                      
-                      //if (line != NULL) g_free (line);
+                      partyline_add_channel (line);
+                      break;
                     }
-                    /* detect whenever we have joined a channel */
-                    if (!strncmp ("has joined", &line[strlen (playernames[playernum])+1], 10))
+                      
+                    if (!strncmp ("List", line, 4))
                     {
-                      partyline_joining_channel (&line[strlen (playernames[playernum])+20]);
+                      partyline_more_channel_lines ();
+                      break;
                     }
-                    else if (!strncmp ("Joined existing Channel", line, 23))
+                      
+                    if (!strncmp ("TetriNET", line, 8))
+                      break;
+                      
+                    if (!strncmp ("You do NOT", line, 10))
                     {
-                      partyline_joining_channel (&line[26]);
-                    }
-                    else if (!strncmp ("Created new Channel", line, 19))
-                    {
-                      partyline_joining_channel (&line[22]);
+                      /* we will use the error message as list stopper */
+                      list_issued--;
+                      if (list_issued <= 0)
+                        stop_list();
+                      break;
                     }
                     
+                    if (!strncmp ("Use", line, 3))
+                      break;
+                      
+                    //if (line != NULL) g_free (line);
+                  }
+                  /* detect whenever we have joined a channel */
+                  if (!strncmp ("has joined", &line[strlen (playernames[playernum])+1], 10))
+                  {
+                    partyline_joining_channel (&line[strlen (playernames[playernum])+20]);
+                  }
+                  else if (!strncmp ("Joined existing Channel", line, 23))
+                  {
+                    partyline_joining_channel (&line[26]);
+                  }
+                  else if (!strncmp ("Created new Channel", line, 19))
+                  {
+                    partyline_joining_channel (&line[22]);
+                  }
+                  else if (!strncmp ("You have joined", line, 15))
+                  {
+                    partyline_joining_channel (&line[16]);
+                  }
+                  
+                  aux = g_strconcat ("You tell ", playernames[playernum], ": --- MARK ---", NULL);
+                  if (!strcmp (aux, line))
+                  {
+                    list_issued--;
+                    if (list_issued <= 0)
+                      stop_list ();
+                    break;
+                  }
+                    
+                  if (tetrix) {
                     g_snprintf (buf, sizeof(buf), "*** %s", token);
                     partyline_text (buf);
                     break;
+                  }
+                  else
+                    plinemsg ("Server", token);
                 }
-                else
-                  plinemsg ("Server", token);
             }
             else
             {
@@ -467,6 +494,8 @@ void tetrinet_inmessage (enum inmsg_type msgtype, char *data)
                   if (list_issued <= 0)
                     stop_list();
                 }
+                else
+                  plinemsg (playernames[pnum], token);
                 //g_free (line);
               }
               else
@@ -575,6 +604,7 @@ void tetrinet_inmessage (enum inmsg_type msgtype, char *data)
             commands_checkstate ();
             partyline_fmt (_("%c*** The game has %cstarted"),
                            TETRI_TB_C_BRIGHT_RED, TETRI_TB_BOLD);
+            show_stop_button ();
             /* switch to playerfields when game starts */
             show_fields_page ();
         }
@@ -600,6 +630,7 @@ void tetrinet_inmessage (enum inmsg_type msgtype, char *data)
             commands_checkstate ();
             partyline_fmt(_("%c*** The game is %cin progress"),
                           TETRI_TB_C_BRIGHT_RED, TETRI_TB_BOLD);;
+            show_stop_button ();
         }
         break;
     case IN_PAUSE:
@@ -629,6 +660,7 @@ void tetrinet_inmessage (enum inmsg_type msgtype, char *data)
         commands_checkstate ();
         partyline_fmt (_("%c*** The game has %cended"),
                        TETRI_TB_C_BRIGHT_RED, TETRI_TB_BOLD);
+        show_start_button ();
         /* go back to partyline when game ends */
         show_partyline_page ();
         break;
@@ -1678,6 +1710,7 @@ notfieldkey:
     if (gdk_keyval_to_lower (keyval) == keys[K_GAMEMSG]) {
         fields_gmsginput (TRUE);
         gmsgstate = 1;
+        tetris_drawcurrentblock ();
         return TRUE;
     }
     if (paused || !playing) return FALSE;
@@ -1686,17 +1719,25 @@ notfieldkey:
         if (!nextblocktimeout)
             sound_playsound (S_ROTATE);
         tetris_blockrotate (1);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_ROTLEFT]) {
         if (!nextblocktimeout)
             sound_playsound (S_ROTATE);
         tetris_blockrotate (-1);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_RIGHT]) {
         tetris_blockmove (1);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_LEFT]) {
         tetris_blockmove (-1);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_DOWN]) {
         if (!downpressed) {
@@ -1704,6 +1745,8 @@ notfieldkey:
             downpressed = 1;
             tetrinet_settimeout (DOWNDELAY);
         }
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_DROP]) {
         int sound;
@@ -1716,22 +1759,46 @@ notfieldkey:
             else sound_playsound (S_DROP);
             tetrinet_sendfield (0);
         }
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_DISCARD]) {
         tetrinet_specialkey(-1);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
-    else switch (keyval) {
-    case GDK_1: tetrinet_specialkey(1); break;
-    case GDK_2: tetrinet_specialkey(2); break;
-    case GDK_3: tetrinet_specialkey(3); break;
-    case GDK_4: tetrinet_specialkey(4); break;
-    case GDK_5: tetrinet_specialkey(5); break;
-    case GDK_6: tetrinet_specialkey(6); break;
-    default:
-        return FALSE;
+    else if (gdk_keyval_to_lower (keyval) == keys[K_SPECIAL1]) {
+	tetrinet_specialkey(1);
+        tetris_drawcurrentblock ();
+        return TRUE;
+    }
+    else if (gdk_keyval_to_lower (keyval) == keys[K_SPECIAL2]) {
+	tetrinet_specialkey(2);
+        tetris_drawcurrentblock ();
+        return TRUE;
+    }
+    else if (gdk_keyval_to_lower (keyval) == keys[K_SPECIAL3]) {
+	tetrinet_specialkey(3);
+        tetris_drawcurrentblock ();
+        return TRUE;
+    }
+    else if (gdk_keyval_to_lower (keyval) == keys[K_SPECIAL4]) {
+	tetrinet_specialkey(4);
+        tetris_drawcurrentblock ();
+        return TRUE;
+    }
+    else if (gdk_keyval_to_lower (keyval) == keys[K_SPECIAL5]) {
+	tetrinet_specialkey(5);
+        tetris_drawcurrentblock ();
+        return TRUE;
+    }
+    else if (gdk_keyval_to_lower (keyval) == keys[K_SPECIAL6]) {
+	tetrinet_specialkey(6);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     tetris_drawcurrentblock ();
-    return TRUE;
+    return FALSE;
 }
 
 void tetrinet_upkey (int keyval)
